% Demo script for SICA algorithm.
% 
% Run script to start demo.
%
% Copyright (C) Morten Mrup, Kristoffer Hougaard Madsen and Technical University of Denmark, 
% March 2007

% Generate data for SICA algorithm
clear all;
maxdelay=25;
usecomp=[1 2 3];
sigmaN=0; % noise variance

% Generate sources
xx=linspace(0,3,1000);
gauswind=exp(-linspace(-2,2,length(xx)).^2/1);
gauswind2=exp(-linspace(0,3,length(xx))/1);
S(1,:)=gauswind.*sin(1.5*2*pi*xx)+ gauswind.*sin(3*2*pi*xx); % low frequency oscillation
S(2,:)=gauswind.*sin(0.9*2*pi*xx)+gauswind.*sin(1*2*pi*xx); % High frequency oscillation
S(3,:)=gauswind.*sin(0.3*2*pi*xx)+gauswind.*sin(2*2*pi*xx); % High frequency oscillation

% Generate mixing and delays
A=rand(10,length(usecomp))-0.5;
A=A+0.2*sign(A);
T=maxdelay*2*(rand(size(A))-0.5); 

% Align S and T
N=size(S,2);
f=j*2*pi*[0:N-1]/N;
tmean=mean(T);
T=T-repmat(tmean,[size(T,1),1]);
Sf=fft(S,[],2);
Sf=Sf(:,1:floor(size(Sf,2)/2)+1);
f=f(1:size(Sf,2));
Sf=Sf.*exp(tmean'*f);
Sft=[Sf conj(Sf(:,end-1:-1:2))];
S=real(ifft(Sft,[],2));

% Generate synthetic data from factors
Sf=fft(S,[],2);
Sf=Sf(:,1:floor(size(Sf,2)/2)+1);
N=size(S,2);
f=j*2*pi*[0:N-1]/N;
f=f(1:size(Sf,2));
for i=1:size(A,1)
   Sft=Sf(usecomp,:).*exp(T(i,usecomp)'*f); 
   Sft=[Sft conj(Sft(:,end-1:-1:2))];
   St=real(ifft(Sft,[],2));
   X(i,:)=A(i,usecomp)*St;
end
Strue=S(usecomp,:);
Atrue=A(:,usecomp);
Ttrue=T(:,usecomp);

% Add noise
N=sigmaN*randn(size(X));
X=X+N;


% Set algorithm parameters
opts.maxiter=100;
[A, S, T, At,St, Tt]=SICA(X,length(usecomp),opts);

% Plot true components
ind=[1 2 3];
noc=size(A,2);
figure;
for k=1:noc
    subplot(noc,3,(k-1)*3+1)
    hinton(Atrue(:,k));
    title(['True A_' num2str(k)]);
    axis tight;
   
    subplot(noc,3,(k-1)*3+2)
    plot(Strue(k,:));
    title(['True S_' num2str(k)]);
    axis tight;

    subplot(noc,3,(k-1)*3+3)
    barh(Ttrue(:,k));
    title(['True T_' num2str(k)]);
    axis tight;    
end

% Plot Estimated components using SICA
figure;
for k=1:noc
    subplot(noc,3,(k-1)*3+1)
    hinton(A(:,ind(k)));
    title(['Est. A_' num2str(k)]);
    axis tight;
   
    subplot(noc,3,(k-1)*3+2)
    plot(S(ind(k),:));
    title(['Est. S_' num2str(k)]);
    axis tight;

    subplot(noc,3,(k-1)*3+3)
    barh(T(:,ind(k)));
    title(['Est. T_' num2str(k)]);
    axis tight; 
end

% Plot ambiguous components found by SISA preprocessing
figure;
for k=1:noc
    subplot(noc,3,(k-1)*3+1)
    hinton(At(:,ind(k)));
    title(['Amb. Est. A_' num2str(k)]);
    axis tight;
   
    subplot(noc,3,(k-1)*3+2)
    plot(St(ind(k),:));
    title(['Amb. Est. S_' num2str(k)]);
    axis tight;

    subplot(noc,3,(k-1)*3+3)
    barh(Tt(:,ind(k)));
    title(['Amb. Est. T_' num2str(k)]);
    axis tight;
end